@@ -30,7 +30,7 @@ jomo1cat.MCMCchain <-
         n.patterns<-nrow(miss.pat)-1
       }
     } else {
-      miss.pat<-matrix(0,2,ncol(Y.cat)+1)
+      miss.pat<-matrix(0,2,ncol(Y.cat))
       n.patterns<-nrow(miss.pat)-1
     }
     
