@@ -88,7 +88,7 @@ function(Y.con, Y.cat, Y.numcat, X=NULL, Z=NULL, clus, beta.start=NULL, u.start=
       n.patterns<-nrow(miss.pat)-1
     }
   } else {
-    miss.pat<-matrix(0,2,ncol(Y)+1)
+    miss.pat<-matrix(0,2,ncol(Y))
     n.patterns<-nrow(miss.pat)-1
   }
   
